@@ -3,21 +3,44 @@
 
 #
 # Copyright (c) 2010 Benoît HERVIER
+# Copyright (c) 2011 Neal H. Walfield
 # Licenced under GPLv3
 
 '''A simple Twitter client made with pyqt4 : QListView'''
 
 from list_model import IDROLE, SCREENNAMEROLE, REPLYTEXTROLE, \
-    RETWEETOFROLE, TIMESTAMPROLE, REPLYTOSCREENNAMEROLE
+    RETWEETOFROLE, TIMESTAMPROLE, REPLYTOSCREENNAMEROLE, ISNEWROLE
 
 import os
+import time
+import logging
 
 from PySide.QtGui import QStyledItemDelegate, QListView, QColor, \
     QAbstractItemView, QFontMetrics, QFont, QStyle, QPixmap
 from PySide.QtCore import Qt, QSize, QSettings
 
-from settings import KhweeteurPref
+from theme import DEFAULTTHEME, WHITETHEME, \
+                     COOLWHITETHEME, COOLGRAYTHEME, \
+                     MINITHEME
 
+def to_str(s):
+    """
+    Given a string, do our best to turn it into a unicode compatible
+    object.
+    """
+    if s is None:
+        return ''
+
+    if issubclass(s.__class__, unicode):
+        # It's already unicode, no problem.
+        return s
+
+    # It's not unicode.  Convert it to a unicode string.
+    try:
+        return unicode(s)
+    except UnicodeEncodeError:
+        logging.exception("Failed to convert '%s' to unicode" % s)
+        return unicode(s, errors='replace')
 
 class DefaultCustomDelegate(QStyledItemDelegate):
 
@@ -32,6 +55,8 @@ class DefaultCustomDelegate(QStyledItemDelegate):
 
         self.bg_color = QColor('#000000')
         self.bg_alternate_color = QColor('#333333')
+        self.new_bg_color = QColor('#0044dd')
+        self.new_bg_alternate_color = QColor('#223399')
         self.user_color = QColor('#7AB4F5')
         self.time_color = QColor('#7AB4F5')
         self.replyto_color = QColor('#7AB4F5')
@@ -69,8 +94,7 @@ class DefaultCustomDelegate(QStyledItemDelegate):
         try:
             return self.memoized_size[uid]
         except:
-            size = QStyledItemDelegate.sizeHint(self, option, index)
-            tweet = index.data(Qt.DisplayRole)
+            tweet = to_str(index.data(Qt.DisplayRole))
 
             # One time is enought sizeHint need to be fast
 
@@ -83,7 +107,7 @@ class DefaultCustomDelegate(QStyledItemDelegate):
             if not self.minifm:
                 self.miniFont = QFont(option.font)
                 self.miniFont.setPointSizeF(option.font.pointSizeF()
-                        * 0.80000000000000004 * self.fsize)
+                        * 0.8 * self.fsize)
                 self.minifm = QFontMetrics(self.miniFont)
 
             height = self.fm.boundingRect(
@@ -95,7 +119,7 @@ class DefaultCustomDelegate(QStyledItemDelegate):
                 tweet,
                 ).height()
 
-            reply_text = index.data(role=REPLYTEXTROLE)
+            reply_text = to_str(index.data(role=REPLYTEXTROLE))
             if reply_text:
                 height += self.minifm.boundingRect(
                     0,
@@ -119,7 +143,7 @@ class DefaultCustomDelegate(QStyledItemDelegate):
 
             if height < 70:
                 height = 70
-            self.memoized_size[uid] = QSize(size.width(), height)
+            self.memoized_size[uid] = QSize(option.rect.width(), height)
             return self.memoized_size[uid]
 
     def paint(
@@ -146,25 +170,34 @@ class DefaultCustomDelegate(QStyledItemDelegate):
         if not self.minifm:
             self.miniFont = QFont(option.font)
             self.miniFont.setPointSizeF(option.font.pointSizeF()
-                                        * 0.80000000000000004 * self.fsize)
+                                        * 0.8 * self.fsize)
 
         # Query data
 
-        tweet = index.data(Qt.DisplayRole)
-        screenname = index.data(SCREENNAMEROLE)
+        tweet = to_str(index.data(Qt.DisplayRole))
+        screenname = to_str(index.data(SCREENNAMEROLE))
         retweet_of = index.data(RETWEETOFROLE)
-        timestamp = index.data(role=TIMESTAMPROLE)
-        reply_name = index.data(role=REPLYTOSCREENNAMEROLE)
-        reply_text = index.data(role=REPLYTEXTROLE)
+        timestamp = to_str(index.data(role=TIMESTAMPROLE))
+        reply_name = to_str(index.data(role=REPLYTOSCREENNAMEROLE))
+        reply_text = to_str(index.data(role=REPLYTEXTROLE))
+        is_new = index.data(role=ISNEWROLE)
 
         painter.save()
 
         # Draw alternate ?
 
         if index.row() % 2 == 0:
-            painter.fillRect(option.rect, self.bg_color)
+            if is_new:
+                color = self.new_bg_color
+            else:
+                color = self.bg_color
         else:
-            painter.fillRect(option.rect, self.bg_alternate_color)
+            if is_new:
+                color = self.new_bg_alternate_color
+            else:
+                color = self.bg_alternate_color
+
+        painter.fillRect(option.rect, color)
 
         # highlight selected items
 
@@ -348,15 +381,15 @@ class KhweetsView(QListView):
     def refreshCustomDelegate(self):
         settings = QSettings('Khertan Software', 'Khweeteur')
         theme = settings.value('theme')
-        if theme == KhweeteurPref.WHITETHEME:
+        if theme == WHITETHEME:
             self.custom_delegate = WhiteCustomDelegate(self)
-        elif theme == KhweeteurPref.DEFAULTTHEME:
+        elif theme == DEFAULTTHEME:
             self.custom_delegate = DefaultCustomDelegate(self)
-        elif theme == KhweeteurPref.COOLWHITETHEME:
+        elif theme == COOLWHITETHEME:
             self.custom_delegate = CoolWhiteCustomDelegate(self)
-        elif theme == KhweeteurPref.COOLGRAYTHEME:
+        elif theme == COOLGRAYTHEME:
             self.custom_delegate = CoolGrayCustomDelegate(self)
-        elif theme == KhweeteurPref.MINITHEME:
+        elif theme == MINITHEME:
             self.custom_delegate = MiniDefaultCustomDelegate(self)
         else:
             self.custom_delegate = DefaultCustomDelegate(self)
