@@ -3,13 +3,14 @@
 
 #
 # Copyright (c) 2010 Benoît HERVIER
+# Copyright (c) 2011 Neal H. Walfield
 # Licenced under GPLv3
 
 '''A Twitter client made with Python and Qt'''
 
 from __future__ import with_statement
 
-__version__ = '0.5.27'
+__version__ = '0.7.1'
 
 # import sip
 # sip.setapi('QString', 2)
@@ -18,7 +19,8 @@ __version__ = '0.5.27'
 from PySide.QtGui import QMainWindow, QHBoxLayout, QSizePolicy, QToolButton, \
     QVBoxLayout, QFileDialog, QDesktopServices, QScrollArea, QPushButton, \
     QToolBar, QLabel, QWidget, QInputDialog, QMenu, QAction, QApplication, \
-    QIcon, QMessageBox, QPlainTextEdit, QTextCursor
+    QIcon, QMessageBox, QPlainTextEdit, QTextCursor, QDialog, QCheckBox, \
+    QDialogButtonBox
 from PySide.QtCore import Qt, QUrl, QSettings, Slot, Signal, QTimer
 
 try:
@@ -35,14 +37,17 @@ from qbadgebutton import QToolBadgeButton
 
 import os
 import sys
+import logging
+import logging_config
 from list_view import KhweetsView
-from list_model import KhweetsModel, ISMEROLE, IDROLE, ORIGINROLE, SCREENNAMEROLE, PROTECTEDROLE, USERIDROLE
+from list_model import KhweetsModel, ISMEROLE, IDROLE, ORIGINROLE, SCREENNAMEROLE, PROTECTEDROLE, USERIDROLE, ISNEWROLE
+from posttweet import post_tweet
+from settings import settings_db, accounts
 
-
-try:
-    from QtMobility.Location import QGeoPositionInfoSource
-except:
-    print 'Pyside QtMobility not installed or broken'
+import dbus
+import dbus.mainloop.glib
+dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
+from dbushandler import KhweeteurDBusHandler
 
 class KhweeteurAbout(QMainWindow):
 
@@ -169,10 +174,6 @@ class Khweeteur(QApplication):
         self.setOrganizationDomain('khertan.net')
         self.setApplicationName('Khweeteur')
 
-        try:
-            self.osso_c = osso.Context('net.khertan.khweeteur', "0.0.1", False)
-        except:
-            pass
         self.run()
 
     def check_crash_report(self):
@@ -182,7 +183,7 @@ class Khweeteur(QApplication):
             import pickle
             if (( QMessageBox.question(None,
                 "Khweeteur Crash Report",
-                "An error occur on Khweeteur in the previous launch. Report this bug on the bug tracker ?",
+                "An error occured during your last execution of Khweeteur. Send relevant information to the bug tracker?",
                 QMessageBox.Yes| QMessageBox.Close)) ==  QMessageBox.Yes):
                 url = 'http://khertan.net/report.php' # write ur URL here
                 try:
@@ -205,7 +206,7 @@ class Khweeteur(QApplication):
                     print detail
                     QMessageBox.question(None,
                     "Khweeteur Crash Report",
-                    "An error occur during the report : %s" % detail,
+                    "An error occurred sending the report: %s" % detail,
                     QMessageBox.Close)
                     return False
 
@@ -236,6 +237,11 @@ class KhweeteurWin(QMainWindow):
     activated_by_dbus = Signal()
 
     def __init__(self, parent=None):
+        # Ensure that we are able to get our DBus name before we
+        # create the window.
+        self.dbus_handler = KhweeteurDBusHandler(self)
+        self.dbus_handler.attach_win(self)
+
         QMainWindow.__init__(self, parent)
 
         try:
@@ -252,10 +258,6 @@ class KhweeteurWin(QMainWindow):
         self.setCentralWidget(self.view)
 
         self.model = KhweetsModel()
-        try:
-            self.model.setLimit(int(settings.value('tweetHistory')))
-        except:
-            self.model.setLimit(60)
         self.view.setModel(self.model)
         self.view.clicked.connect(self.switch_tb_action)
 
@@ -354,32 +356,36 @@ class KhweeteurWin(QMainWindow):
         self.toolbar.addAction(self.tb_fullscreen)
         self.list_tb_action.append(self.tb_fullscreen)
 
-        self.setWindowTitle('Khweeteur : Home')
+        self.setWindowTitle('Khweeteur: Home')
 
         self.show()
 
-        self.model.load('HomeTimeline',limit=5)
         QTimer.singleShot(0,self.post_init_2)
         QTimer.singleShot(0,self.post_init_3)
         QApplication.processEvents()
 
     @Slot()
     def post_init_3(self):
-        self.model.load('HomeTimeline')
+        self.do_model_load('HomeTimeline')
         #Check if there is at least one account
-        nb_accounts = QSettings().beginReadArray('accounts')
-        if not nb_accounts:
+        if not accounts():
             if (( QMessageBox.question(None,
                 "Khweeteur",
-                'None microblogging account found, would you want to add one now in preferences ?',
+                'No microblogging account configured, do you want to add one now?',
                 QMessageBox.Yes| QMessageBox.Close)) ==  QMessageBox.Yes):
                     self.showPrefs()
 
     @Slot()
     def post_init_2(self):
         self.listen_dbus()
-        self.dbus_handler.require_update()
-        self.tb_update.triggered.connect(self.dbus_handler.require_update)
+
+        QTimer.singleShot(3 * 1000, self.dbus_handler.require_update)
+
+        # When the user requests an update, mark it as mandatory (=
+        # not optional).
+        self.tb_update.triggered.connect(
+            lambda: self.dbus_handler.require_update(optional=False))
+
         self.geolocDoStart()
 
         self.loadSearchMenu()
@@ -526,6 +532,32 @@ class KhweeteurWin(QMainWindow):
         self.tb_new.triggered.connect(self.switch_tb_edit)
 
 
+    def do_model_load(self, *args, **kwargs):
+        if not self.model.load(*args, **kwargs):
+            # It was a reload.  Don't scroll.
+            return
+
+        def test(i):
+            try:
+                return not self.model.data(i, ISNEWROLE)
+            except IndexError:
+                return False
+
+        def bisect_right(test, x):
+            lo = 0
+            hi = self.model.rowCount()
+            while lo < hi:
+                mid = (lo+hi)//2
+                if x < test(mid): hi = mid
+                else: lo = mid+1 
+            return lo
+
+        # We want to scroll to the last new message (i - 1),
+        # not the first old message (i)
+        first_old = bisect_right(test, False)
+        # print "Scrolling to %d" % (first_old,)
+        self.view.scrollTo(self.model.index(max (0,  first_old - 1), 0))
+
     @Slot()
     def do_tb_fullscreen(self):
         if self.isFullScreen():
@@ -551,29 +583,27 @@ class KhweeteurWin(QMainWindow):
         self.model.refreshTimestamp()
 
     def listen_dbus(self):
-        import dbus
-        import dbus.service
-        from dbushandler import KhweeteurDBusHandler
-        import dbus.mainloop.glib
-        #from dbus.mainloop.qt import DBusQtMainLoop
-        #self.dbus_loop = DBusQtMainLoop()
-        dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
-        #dbus.set_default_main_loop(self.dbus_loop)
         self.bus = dbus.SessionBus()
 
         # Connect the new tweet signal
 
-        self.bus.add_signal_receiver(self.new_tweets,
-                                     path='/net/khertan/Khweeteur',
-                                     dbus_interface='net.khertan.Khweeteur',
-                                     signal_name='new_tweets')
+        self.bus.add_signal_receiver(
+            self.new_tweets,
+            path='/net/khertan/khweeteur/daemon',
+            dbus_interface='net.khertan.khweeteur.daemon',
+            signal_name='new_tweets')
         self.bus.add_signal_receiver(self.stop_spinning,
-                                     path='/net/khertan/Khweeteur',
-                                     dbus_interface='net.khertan.Khweeteur',
+                                     path='/net/khertan/khweeteur',
+                                     dbus_interface='net.khertan.khweeteur',
                                      signal_name='refresh_ended')
-        self.dbus_handler = KhweeteurDBusHandler(self)
-        self.dbus_handler.attach_win(self)
-        self.activated_by_dbus.connect(self.activateWindow)
+        self.activated_by_dbus.connect(self.bring_to_front)
+
+    def bring_to_front(self):
+        try:
+            self.activateWindow()
+            self.raise_()
+        except Exception:
+            logging.exception("bring to front")
 
     def stop_spinning(self):
         try:
@@ -605,7 +635,7 @@ class KhweeteurWin(QMainWindow):
             self.tb_list_button.update()
 
         if self.model.call == msg:
-            self.model.load(msg)
+            self.do_model_load(msg)
 
     @Slot()
     def show_search(self):
@@ -617,8 +647,7 @@ class KhweeteurWin(QMainWindow):
         self.mention_button.setChecked(False)
         self.tb_list_button.setChecked(False)
         self.near_button.setChecked(False)
-        self.view.scrollToTop()
-        self.model.load('Search:' + terms)
+        self.do_model_load('Search:' + terms)
         self.delete_search_action.setVisible(True)
         self.setWindowTitle('Khweeteur : ' + terms)
 
@@ -635,13 +664,20 @@ class KhweeteurWin(QMainWindow):
         self.tb_list_button.setChecked(True)
         self.mention_button.setChecked(False)
         self.near_button.setChecked(False)
-        self.view.scrollToTop()
-        self.model.load('List:' + user + ':' + tid)
+        self.do_model_load('List:' + user + ':' + tid)
         self.delete_search_action.setVisible(True)
         self.setWindowTitle('Khweeteur List : ' + name)
 
     @Slot()
     def show_hometimeline(self):
+        # The user just clicked on the home button.  If the home
+        # timeline is already active, the button was checked and now
+        # it is unchecked.  Otherwise, another view was active and now
+        # the home button is checked.
+        if not self.home_button.isChecked():
+            # Already showing this.  Reset the new messages.
+            self.model.reset_new_message_horizon()
+
         self.home_button.setCounter(0)
         self.home_button.setChecked(True)
         self.msg_button.setChecked(False)
@@ -649,8 +685,7 @@ class KhweeteurWin(QMainWindow):
         self.near_button.setChecked(False)
         self.tb_list_button.setChecked(False)
         self.mention_button.setChecked(False)
-        self.view.scrollToTop()
-        self.model.load('HomeTimeline')
+        self.do_model_load('HomeTimeline')
         self.setWindowTitle('Khweeteur : Home')
         self.delete_search_action.setVisible(False)
 
@@ -716,13 +751,13 @@ class KhweeteurWin(QMainWindow):
             filename = filename[0]
 
         if filename:
-            self.dbus_handler.post_tweet(  # shorten_url=\
-                                           # serialize=\
-                                           # text=\
-                                           # lattitude =
-                                           # longitude =
-                                           # base_url =
-                                           # tweet_id =
+            post_tweet(  # shorten_url=\
+                         # serialize=\
+                         # text=\
+                         # latitude =
+                         # longitude =
+                         # base_url =
+                         # tweet_id =
                 0,
                 1,
                 text,
@@ -733,7 +768,7 @@ class KhweeteurWin(QMainWindow):
                 '',
                 )
             self.switch_tb_default()
-            self.dbus_handler.require_update()
+            self.dbus_handler.require_update(only_uploads=True)
 
     @Slot()
     def do_tb_openurl(self):
@@ -748,27 +783,134 @@ class KhweeteurWin(QMainWindow):
             except:
                 raise
 
+    def select_accounts(self, message='Select Accounts',
+                        default_accounts=None, service=None):
+        """
+        Ask the user what accounts should be used for some action,
+        which is described by message.
+
+        If default_accounts is not None, it must be a list of account
+        identifiers that should be selected by default.  Otherwise,
+        the default is those accounts with 'use_for_tweet' set to
+        'true'.
+
+        If service is not None, only asks about those accounts for the
+        specified service.
+
+        The make default button is only shown if default_accoutns and
+        service is None.
+
+        Returns account identifiers (use as the base_url).
+        """
+        print("Default accounts: %s; service: %s"
+              % (str (default_accounts), str(service)))
+
+        def make_default():
+            for account_widget in account_widgets:
+                checkbox = account_widget['box']
+                account = account_widget['account']
+
+                if checkbox.checkState() == Qt.Checked:
+                    account.use_for_tweet = 'true'
+                else:
+                    account.use_for_tweet = 'false'
+
+            accounts(True)
+
+        accounts_to_consider = []
+        for account in accounts():
+            if service is None or service == account['base_url']:
+                accounts_to_consider.append(account)
+
+        target_accounts = accounts_to_consider
+
+        if len(accounts_to_consider) == 1:
+            # We have exactly one account that is marked as
+            # appropriate for sending tweets.  Use it.
+            pass
+        else:
+            d = QDialog()
+            d.setWindowTitle(message)
+
+            layout = QVBoxLayout()
+            account_widgets = []
+            for account in accounts_to_consider:
+                checkbox = QCheckBox(account['name'])
+                account_widget = {'box': checkbox, 'account': account}
+                account_widgets.append(account_widget)
+
+                checked = False
+                if default_accounts is not None:
+                    if account.uuid in default_accounts:
+                        checked = True
+                else:
+                    if account.use_for_tweet == 'true':
+                        checked = True
+
+                checkbox.setCheckState(Qt.Checked if checked else Qt.Unchecked)
+
+                layout.addWidget(checkbox)
+
+            buttonbox = QDialogButtonBox(
+                QDialogButtonBox.Ok|QDialogButtonBox.Cancel,
+                parent=d)
+            buttonbox.accepted.connect(d.accept)
+            buttonbox.rejected.connect(d.reject)
+
+            if default_accounts is None and service is None:
+                make_default_button = QPushButton("Make &Default")
+                buttonbox.addButton(make_default_button,
+                                    QDialogButtonBox.ApplyRole)
+                make_default_button.clicked.connect(make_default)
+
+            layout.addWidget(buttonbox)
+
+            d.setLayout(layout)
+            if d.exec_() == QDialog.Accepted:
+                target_accounts \
+                    = [account_widget['account']
+                       for account_widget in account_widgets
+                       if account_widget['box'].checkState() == Qt.Checked]
+            else:
+                target_accounts = []
+
+        base_urls = [account['base_url'] + ';' + account['token_key']
+                     for account in target_accounts]
+
+        return base_urls
+
     @Slot()
     def do_tb_send(self):
         is_not_reply = self.tb_text_reply_id == 0
-        self.dbus_handler.post_tweet(  # shorten_url=\
-                                       # serialize=\
-                                       # text=\
-                                       # lattitude =
-                                       # longitude =
-                                       # base_url
-                                       # action
-            1,
-            1,
-            self.tb_text.toPlainText(),
-            ('' if self.geoloc_source == None else self.geoloc_coordinates[0]),
-            ('' if self.geoloc_source == None else self.geoloc_coordinates[1]),
-            ('' if is_not_reply else self.tb_text_reply_base_url),
-            ('tweet' if is_not_reply else 'reply'),
-            ('' if is_not_reply else str(self.tb_text_reply_id)),
-            )
+
+        default_accounts = None
+        if not is_not_reply:
+            default_accounts = [self.tb_text_reply_base_url]
+
+        base_urls = self.select_accounts(default_accounts=default_accounts)
+        if not base_urls:
+            # Nothing selected => cancelled.
+            return
+
+        for base_url in base_urls:
+            post_tweet(  # shorten_url=\
+                         # serialize=\
+                         # text=\
+                         # latitude =
+                         # longitude =
+                         # base_url
+                         # action
+                1,
+                1,
+                self.tb_text.toPlainText(),
+                ('' if self.geoloc_source == None else self.geoloc_coordinates[0]),
+                ('' if self.geoloc_source == None else self.geoloc_coordinates[1]),
+                base_url,
+                ('tweet' if is_not_reply else 'reply'),
+                ('' if is_not_reply else str(self.tb_text_reply_id)),
+                )
         self.switch_tb_default()
-        self.dbus_handler.require_update()
+        self.dbus_handler.require_update(only_uploads=True)
 
     @Slot()
     def do_tb_reply(self):
@@ -814,34 +956,35 @@ class KhweeteurWin(QMainWindow):
             if self.model.data(index, role=PROTECTEDROLE):
                 screenname = self.model.data(index, role=SCREENNAMEROLE)
                 QMessageBox.warning(self, 'Khweeteur - Retweet',
-                                    "%s protect his tweets you can't retweet them"
+                                    "%s's tweets are protected: you can't retweet them."
                                      % screenname, QMessageBox.Close)
 
         if not (( QMessageBox.question(None,
             "Khweeteur Retweet",
-            "Did you want to retweet '%s'?" % tweet_text,
+            "Do you want to retweet '%s'?" % tweet_text,
             QMessageBox.Yes| QMessageBox.Close)) ==  QMessageBox.Yes):
-            tweet_id = None
+            return
 
-        if tweet_id:
-            self.dbus_handler.post_tweet(  # shorten_url=\
-                                           # serialize=\
-                                           # text=\
-                                           # lattitude =
-                                           # longitude =
-                                           # base_url =
-                                           # tweet_id =
+        for base_url in self.select_accounts(default_accounts=[tweet_source]):
+            post_tweet(  # shorten_url=\
+                         # serialize=\
+                         # text=\
+                         # latitude =
+                         # longitude =
+                         # base_url =
+                         # action =
+                         # tweet_id =
                 0,
                 0,
                 '',
                 '',
                 '',
-                tweet_source,
+                base_url,
                 'retweet',
                 str(tweet_id),
                 )
             self.switch_tb_default()
-            self.dbus_handler.require_update()
+            self.dbus_handler.require_update(only_uploads=True)
 
     @Slot()
     def do_tb_delete(self):
@@ -853,18 +996,19 @@ class KhweeteurWin(QMainWindow):
 
         if not (( QMessageBox.question(None,
             "Khweeteur Delete",
-            "Did you really want to delete '%s'?" % tweet_text,
+            "Do you really want to delete '%s'?" % tweet_text,
             QMessageBox.Yes| QMessageBox.Close)) ==  QMessageBox.Yes):
             tweet_id = None
 
         if tweet_id:
-            self.dbus_handler.post_tweet(  # shorten_url=\
-                                           # serialize=\
-                                           # text=\
-                                           # lattitude =
-                                           # longitude =
-                                           # base_url =
-                                           # tweet_id =
+            post_tweet(  # shorten_url=\
+                         # serialize=\
+                         # text=\
+                         # latitude =
+                         # longitude =
+                         # base_url =
+                         # action =
+                         # tweet_id =
                 0,
                 0,
                 '',
@@ -875,7 +1019,7 @@ class KhweeteurWin(QMainWindow):
                 str(tweet_id),
                 )
             self.switch_tb_default()
-            self.dbus_handler.require_update()
+            self.dbus_handler.require_update(only_uploads=True)
 
     @Slot()
     def do_tb_favorite(self):
@@ -887,29 +1031,30 @@ class KhweeteurWin(QMainWindow):
 
         if not (( QMessageBox.question(None,
             "Khweeteur Favorite",
-            "Did you really want to favorite '%s'?" % tweet_text,
+            "Do you really want to favorite '%s'?" % tweet_text,
             QMessageBox.Yes| QMessageBox.Close)) ==  QMessageBox.Yes):
-            tweet_id = None
+            return
 
-        if tweet_id:
-            self.dbus_handler.post_tweet(  # shorten_url=\
-                                           # serialize=\
-                                           # text=\
-                                           # lattitude =
-                                           # longitude =
-                                           # base_url =
-                                           # tweet_id =
+        for base_url in self.select_accounts(default_accounts=[tweet_source]):
+            post_tweet(  # shorten_url=\
+                         # serialize=\
+                         # text=\
+                         # latitude =
+                         # longitude =
+                         # base_url =
+                         # action =
+                         # tweet_id =
                 0,
                 0,
                 '',
                 '',
                 '',
-                tweet_source,
+                base_url,
                 'favorite',
                 str(tweet_id),
                 )
             self.switch_tb_default()
-            self.dbus_handler.require_update()
+            self.dbus_handler.require_update(only_uploads=True)
 
     @Slot()
     def do_tb_follow(self):
@@ -919,31 +1064,35 @@ class KhweeteurWin(QMainWindow):
             tweet_source = self.model.data(index, role=ORIGINROLE)
             screenname = self.model.data(index, role=SCREENNAMEROLE)
 
+        if user_id is None:
+            user_id = screenname
+
         if not (( QMessageBox.question(None,
             "Khweeteur Follow",
-            "Did you really want to follow '%s'?" % screenname,
+            "Do you really want to follow '%s'?" % screenname,
             QMessageBox.Yes| QMessageBox.Close)) ==  QMessageBox.Yes):
-            user_id = None
+            return
 
-        if user_id:
-            self.dbus_handler.post_tweet(  # shorten_url=\
-                                           # serialize=\
-                                           # text=\
-                                           # lattitude =
-                                           # longitude =
-                                           # base_url =
-                                           # tweet_id =
+        for base_url in self.select_accounts(default_accounts=[tweet_source]):
+            post_tweet(  # shorten_url=\
+                         # serialize=\
+                         # text=\
+                         # latitude =
+                         # longitude =
+                         # base_url =
+                         # action =
+                         # tweet_id =
                 0,
                 0,
                 '',
                 '',
                 '',
-                tweet_source,
+                base_url,
                 'follow',
                 str(user_id),
                 )
             self.switch_tb_default()
-            self.dbus_handler.require_update()
+            self.dbus_handler.require_update(only_uploads=True)
 
     @Slot()
     def do_tb_unfollow(self):
@@ -955,18 +1104,19 @@ class KhweeteurWin(QMainWindow):
 
         if not (( QMessageBox.question(None,
             "Khweeteur Unfollow",
-            "Did you really want to unfollow '%s'?" % screenname,
+            "Do you really want to unfollow '%s'?" % screenname,
             QMessageBox.Yes| QMessageBox.Close)) ==  QMessageBox.Yes):
             user_id = None
 
         if user_id:
-            self.dbus_handler.post_tweet(  # shorten_url=\
-                                           # serialize=\
-                                           # text=\
-                                           # lattitude =
-                                           # longitude =
-                                           # base_url =
-                                           # tweet_id =
+            post_tweet(  # shorten_url=\
+                         # serialize=\
+                         # text=\
+                         # latitude =
+                         # longitude =
+                         # base_url =
+                         # action =
+                         # tweet_id =
                 0,
                 0,
                 '',
@@ -977,10 +1127,14 @@ class KhweeteurWin(QMainWindow):
                 str(user_id),
                 )
             self.switch_tb_default()
-            self.dbus_handler.require_update()
+            self.dbus_handler.require_update(only_uploads=True)
 
     @Slot()
     def show_mentions(self):
+        if not self.mention_button.isChecked():
+            # Already showing this.  Reset the new messages.
+            self.model.reset_new_message_horizon()
+
         self.mention_button.setCounter(0)
         self.mention_button.setChecked(True)
         self.msg_button.setChecked(False)
@@ -988,13 +1142,16 @@ class KhweeteurWin(QMainWindow):
         self.tb_search_button.setChecked(False)
         self.home_button.setChecked(False)
         self.near_button.setChecked(False)
-        self.view.scrollToTop()
-        self.model.load('Mentions')
+        self.do_model_load('Mentions')
         self.setWindowTitle('Khweeteur : Mentions')
         self.delete_search_action.setVisible(False)
 
     @Slot()
     def show_dms(self):
+        if not self.msg_button.isChecked():
+            # Already showing this.  Reset the new messages.
+            self.model.reset_new_message_horizon()
+
         self.msg_button.setCounter(0)
         self.msg_button.setChecked(True)
         self.tb_list_button.setChecked(False)
@@ -1002,8 +1159,7 @@ class KhweeteurWin(QMainWindow):
         self.tb_search_button.setChecked(False)
         self.mention_button.setChecked(False)
         self.near_button.setChecked(False)
-        self.view.scrollToTop()
-        self.model.load('DMs')
+        self.do_model_load('DMs')
         self.setWindowTitle('Khweeteur : DMs')
         self.delete_search_action.setVisible(False)
 
@@ -1013,12 +1169,16 @@ class KhweeteurWin(QMainWindow):
         if settings.value('useGPS') != '2':
             if (( QMessageBox.question(None,
                 "Khweeteur",
-                'This feature require the activation of the GPS, would you want to activate it now ?',
+                'This feature requires the GPS, do you want to activate it now?',
                 QMessageBox.Yes| QMessageBox.Close)) ==  QMessageBox.Yes):
                     settings.setValue('useGPS','2')
                     settings.sync()
                     self.geolocDoStart()
-                    self.dbus_handler.require_update()
+                    self.dbus_handler.require_update(only_uploads=True)
+
+        if not self.near_button.isChecked():
+            # Already showing this.  Reset the new messages.
+            self.model.reset_new_message_horizon()
 
         self.near_button.setCounter(0)
         self.near_button.setChecked(True)
@@ -1027,9 +1187,8 @@ class KhweeteurWin(QMainWindow):
         self.home_button.setChecked(False)
         self.tb_search_button.setChecked(False)
         self.mention_button.setChecked(False)
-        self.view.scrollToTop()
-        self.model.load('Nears')
-        self.setWindowTitle('Khweeteur : Near Tweets')
+        self.do_model_load('Nears')
+        self.setWindowTitle('Khweeteur: Nearby Tweets')
         self.delete_search_action.setVisible(False)
 
     @Slot()
@@ -1159,11 +1318,13 @@ class KhweeteurWin(QMainWindow):
         self.geoloc_coordinates = None
         if self.geoloc_source is None:
             try:
+                from QtMobility.Location import QGeoPositionInfoSource
                 self.geoloc_source = \
                     QGeoPositionInfoSource.createDefaultSource(None)
             except:
                 self.geoloc_source = None
-                print 'PySide QtMobility not installed or package broken'
+                logging.exception(
+                    'PySide QtMobility not installed or package broken')
             if self.geoloc_source is not None:
                 self.geoloc_source.setUpdateInterval(50000)
                 self.geoloc_source.positionUpdated.connect(self.geolocUpdated)
@@ -1187,20 +1348,47 @@ class KhweeteurWin(QMainWindow):
             print 'GPS Update not valid'
 
 
-if __name__ == '__main__':
-    from subprocess import Popen
-    Popen(['/usr/bin/python', os.path.join(os.path.dirname(__file__),
-          'daemon.py'), 'start'])
+#Here is the installation of the hook. Each time a untrapped/unmanaged exception will
+#happen my_excepthook will be called.
+def install_excepthook():
+    '''Install exception hook for the bug reporter'''
+
+    def write_report(error):
+        import pickle
+        import os.path
+        filename = os.path.join(os.path.join(os.path.expanduser("~"),'.khweeteur_crash_report'))
+        output = open(filename, 'wb')
+        pickle.dump(error, output)
+        output.close()
+
+    def my_excepthook(exctype, value, tb):
+        #traceback give us all the errors information message like the method, file line ... everything like
+        #we have in the python interpreter
+        import traceback
+        import PySide
+        from qwidget_gui import __version__
+        s = ''.join(traceback.format_exception(exctype, value, tb))
+        print 'Except hook', exctype
+        print 'Except hook called : %s' % (s)
+        formatted_text = "Maemo Khweeteur Version %s\nPySide Version : %s\nQt Version : %s\nPython Trace : %s\n" % ( __version__, repr(PySide.__version_info__), repr(PySide.QtCore.__version_info__),s)
+        write_report(formatted_text)
+
+    sys.excepthook = my_excepthook
+
+def takeScreenShot(app):
+    from PySide.QtGui import QPixmap
+    pvr = "/home/user/.cache/launch/net.khertan.khweeteur.pvr"
+    QPixmap.grabWidget(app.win).save(pvr, 'png') # tell it to grab only your self.centralwidget screen, which is just window screen without the menu status bar on top.
+
+def main():
+    logging_config.init(".khweeteur", debug=True, max_logfiles=5,
+                        program_name='khweeteur')
+    install_excepthook()
+
     app = Khweeteur()
     app.exec_()
-    settings = QSettings('Khertan Software', 'Khweeteur')
-    if settings.contains('useDaemon'):
-        print settings.value('useDaemon')
-        if settings.value('useDaemon') != '2':
-            print 'Stop daemon'
 
-            # use system to wait the exec
+    takeScreenShot(app)
 
-            os.system('/usr/bin/python '
-                      + os.path.join(os.path.dirname(__file__), 'daemon.py')
-                      + ' stop')
+if __name__ == '__main__':
+    main()
